@@ -7,7 +7,7 @@
 #include <libgnomevfs/gnome-vfs-resolve.h>
 #include <libgnomevfs/gnome-vfs-dns-sd.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
-#include <pyorbit.h>
+#include "pygnomevfsbonobo.h"
 
 static GHashTable *monitor_hash;
 static gint monitor_id_counter = 0;
@@ -67,6 +67,14 @@ static PyTypeObject *_PyGObject_Type;
 
 extern void pygvvolume_add_constants(PyObject *m);
 
+static void
+pygvfs_lazy_load_pygnomevfsbonobo(void)
+{
+    static PyObject *pygnomevfs_module = NULL;
+    if (!pygnomevfs_module)
+        pygnomevfs_module = pygnome_vfs_bonobo_init();
+}
+
 
 gboolean
 pygnome_vfs_result_check(GnomeVFSResult result)
@@ -633,12 +641,12 @@ pygvfs_get_mime_type_for_data(PyObject *self, PyObject *args)
 {
     char *data;
     char const *mime;
-    int data_size, data_size1 = -G_MAXINT;
+    Py_ssize_t data_size, data_size1 = PY_SSIZE_T_MIN;
     
     if(!PyArg_ParseTuple(args, "s#|i:gnomevfs.get_mime_type_for_data",
 			 &data, &data_size, &data_size1))
 	return NULL;
-    if (data_size1 != -G_MAXINT)
+    if (data_size1 != PY_SSIZE_T_MIN)
 	PyErr_Warn(PyExc_DeprecationWarning, "ignoring deprecated argument data_size");
     pyg_begin_allow_threads;
     mime = gnome_vfs_get_mime_type_for_data(data, data_size);
@@ -894,10 +902,10 @@ pygvfs_mime_application_new(GnomeVFSMimeApplication *mimeapp)
     for (i = 0, l = mimeapp->supported_uri_schemes; l; ++i, l = l->next)
 	PyList_SET_ITEM(uri_schemes, i, PyString_FromString((const char *) l->data));
 
-    return Py_BuildValue("sssOONO", mimeapp->id, mimeapp->name,
+    return Py_BuildValue("sssOiNO", mimeapp->id, mimeapp->name,
 			 mimeapp->command,
 			 mimeapp->can_open_multiple_files? Py_True : Py_False, 
-			 mimeapp->expects_uris? Py_True : Py_False,
+			 mimeapp->expects_uris,
 			 uri_schemes,
 			 mimeapp->requires_terminal? Py_True : Py_False);
 }
@@ -1173,16 +1181,8 @@ pygvfs_mime_action_new(GnomeVFSMimeAction *action)
 			     pygvfs_mime_application_new(action->action.application));
 
     case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT: {
-	CORBA_any any;
-	PyObject *component;
-	any._type = TC_Bonobo_ServerInfo;
-	any._value = action->action.application;
-	component = pyorbit_demarshal_any(&any);
-	if (!component) {
-	    PyErr_SetString(PyExc_TypeError, "unable to convert Bonobo_ServerInfo of component");
-	    return NULL;
-	}
-	return Py_BuildValue("(iN)", action->action_type, component);
+        pygvfs_lazy_load_pygnomevfsbonobo();
+        return pygnomevfs_bonobo_mime_component_action_new(action);
     }
     default:
 	PyErr_SetString(PyExc_ValueError, "unknown action type returned");
@@ -1210,91 +1210,6 @@ pygvfs_mime_get_default_action(PyObject *self, PyObject *args, PyObject *kwargs)
     return retval;
 }
 
-static PyObject *
-pygvfs_mime_get_default_component(PyObject *self, PyObject *args, PyObject *kwargs)
-{
-    static char *kwlist[] = { "mime_type", NULL };
-    char *mime_type;
-    Bonobo_ServerInfo *component;
-    PyObject *py_component;
-    CORBA_any any;
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s:gnomevfs.mime_get_default_component",
-				     kwlist, &mime_type))
-	return NULL;
-    if (!(component = gnome_vfs_mime_get_default_component(mime_type))) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
-    any._type = TC_Bonobo_ServerInfo;
-    any._value = component;
-    py_component = pyorbit_demarshal_any(&any);
-    if (!component) {
-	PyErr_SetString(PyExc_TypeError, "unable to convert Bonobo_ServerInfo of component");
-	return NULL;
-    }
-    CORBA_free(component);
-    return py_component;
-}
-
-static PyObject *
-pygvfs_mime_components_list_new(GList *list)
-{
-    PyObject *retval;
-    Bonobo_ServerInfo *component;
-    PyObject *py_component;
-    CORBA_any any;
-    guint i, len = g_list_length(list);
-    
-    retval = PyList_New(len);
-    for (i = 0; list; ++i, list = list->next) {
-	g_assert(i < len);
-	component = (Bonobo_ServerInfo *) list->data;
-	any._value = component;
-	py_component = pyorbit_demarshal_any(&any);
-	if (!component) {
-	    PyErr_SetString(PyExc_TypeError, "unable to convert Bonobo_ServerInfo of component");
-	    Py_DECREF(retval);
-	    return NULL;
-	}
-	PyList_SET_ITEM(retval, i, py_component);
-    }
-    return retval;
-}
-
-static PyObject *
-pygvfs_mime_get_short_list_components(PyObject *self, PyObject *args, PyObject *kwargs)
-{
-    static char *kwlist[] = { "mime_type", NULL };
-    char *mime_type;
-    GList *list;
-    PyObject *py_list;
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s:gnomevfs.mime_get_short_list_components",
-				     kwlist, &mime_type))
-	return NULL;
-    list = gnome_vfs_mime_get_short_list_components(mime_type);
-    py_list = pygvfs_mime_components_list_new(list);
-    gnome_vfs_mime_component_list_free(list);
-    return py_list;
-}
-
-static PyObject *
-pygvfs_mime_get_all_components(PyObject *self, PyObject *args, PyObject *kwargs)
-{
-    static char *kwlist[] = { "mime_type", NULL };
-    char *mime_type;
-    GList *list;
-    PyObject *py_list;
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s:gnomevfs.mime_get_all_components",
-				     kwlist, &mime_type))
-	return NULL;
-    list = gnome_vfs_mime_get_all_components(mime_type);
-    py_list = pygvfs_mime_components_list_new(list);
-    gnome_vfs_mime_component_list_free(list);
-    return py_list;
-}
 
 static PyObject *
 pygvfs_mime_applications_list_new(GList *list)
@@ -2039,7 +1954,7 @@ _wrap_gnome_vfs_dns_sd_resolve_sync(PyObject *self, PyObject *args, PyObject *kw
     py_hash = PyDict_New();
     g_hash_table_foreach(hash, (GHFunc) __text_hash_to_dict, py_hash);
     g_hash_table_destroy(hash);
-    retval = Py_BuildValue("Ns#", py_hash, text_raw, text_raw_len);
+    retval = Py_BuildValue("Ns#", py_hash, text_raw, (Py_ssize_t) text_raw_len);
     g_free(text_raw);
     return retval;
 }
@@ -2098,6 +2013,24 @@ _wrap_gnome_vfs_get_default_browse_domains(PyObject *self, PyObject *args, PyObj
     return retval;
 }
 
+
+static PyObject *
+_wrap_gnome_vfs_mime_type_get_equivalence(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = { "mime_type", "base_mime_type", NULL };
+    char *mime_type, *base_mime_type;
+    GnomeVFSMimeEquivalence result;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs,
+				     "ss:gnomevfs.mime_type_get_equivalence", kwlist,
+                                      &mime_type, &base_mime_type))
+	return NULL;
+
+    result = gnome_vfs_mime_type_get_equivalence(mime_type, base_mime_type);
+    return PyInt_FromLong(result);
+}
+
+
 static PyMethodDef pygnomevfs_functions[] = {
     { "create", (PyCFunction)pygvfs_create, METH_VARARGS|METH_KEYWORDS },
     { "get_file_info", (PyCFunction)pygvfs_get_file_info,
@@ -2137,16 +2070,10 @@ static PyMethodDef pygnomevfs_functions[] = {
       METH_VARARGS|METH_KEYWORDS },
     { "mime_get_default_action", (PyCFunction)pygvfs_mime_get_default_action,
       METH_VARARGS|METH_KEYWORDS },
-    { "mime_get_default_component", (PyCFunction)pygvfs_mime_get_default_component,
-      METH_VARARGS|METH_KEYWORDS },
-    { "mime_get_short_list_components", (PyCFunction)pygvfs_mime_get_short_list_components,
-      METH_VARARGS|METH_KEYWORDS },
     { "mime_get_short_list_applications", (PyCFunction)pygvfs_mime_get_short_list_applications,
       METH_VARARGS|METH_KEYWORDS },
     { "mime_get_all_applications", (PyCFunction)pygvfs_mime_get_all_applications,
       METH_VARARGS|METH_KEYWORDS },
-    { "mime_get_all_components", (PyCFunction)pygvfs_mime_get_all_components,
-      METH_VARARGS|METH_KEYWORDS },
     { "mime_set_default_action_type", (PyCFunction)pygvfs_mime_set_default_action_type,
       METH_VARARGS|METH_KEYWORDS },
     { "mime_set_default_application", (PyCFunction)pygvfs_mime_set_default_application,
@@ -2235,6 +2162,9 @@ static PyMethodDef pygnomevfs_functions[] = {
       METH_VARARGS|METH_KEYWORDS},
     { "get_default_browse_domains",
       (PyCFunction)_wrap_gnome_vfs_get_default_browse_domains, METH_NOARGS},
+    { "mime_type_get_equivalence",
+      (PyCFunction)_wrap_gnome_vfs_mime_type_get_equivalence,
+      METH_VARARGS|METH_KEYWORDS},
 
     { NULL, NULL, 0 }
     
@@ -2270,6 +2200,8 @@ register_constants(PyObject *module)
     regconst(FILE_INFO_FIELDS_CTIME);
     regconst(FILE_INFO_FIELDS_SYMLINK_NAME);
     regconst(FILE_INFO_FIELDS_MIME_TYPE);
+    regconst(FILE_INFO_FIELDS_ACCESS);
+    regconst(FILE_INFO_FIELDS_IDS);
     regconst(PERM_SUID);
     regconst(PERM_SGID);
     regconst(PERM_STICKY);
@@ -2290,6 +2222,8 @@ register_constants(PyObject *module)
     regconst(FILE_INFO_FORCE_FAST_MIME_TYPE);
     regconst(FILE_INFO_FORCE_SLOW_MIME_TYPE);
     regconst(FILE_INFO_FOLLOW_LINKS);
+    regconst(FILE_INFO_GET_ACCESS_RIGHTS);
+    regconst(FILE_INFO_NAME_ONLY);
     regconst(SET_FILE_INFO_NONE);
     regconst(SET_FILE_INFO_NAME);
     regconst(SET_FILE_INFO_PERMISSIONS);
@@ -2371,6 +2305,9 @@ register_constants(PyObject *module)
     regconst(PRIORITY_MIN);
     regconst(PRIORITY_MAX);
     regconst(PRIORITY_DEFAULT);
+    regconst(MIME_UNRELATED);
+    regconst(MIME_IDENTICAL);
+    regconst(MIME_PARENT);
 
 #undef regconst
 }
@@ -2479,7 +2416,7 @@ pygnomefs_register_gobject_based_classes(PyObject *d)
 }
 
 DL_EXPORT(void)
-initgnomevfs(void)
+init_gnomevfs(void)
 {
     PyObject *m, *d, *o;
 
@@ -2490,7 +2427,6 @@ initgnomevfs(void)
     PyGnomeVFSHandle_Type.ob_type = &PyType_Type;
 
     init_pygobject();
-    init_pyorbit();
     if (!gnome_vfs_init()) {
 	PyErr_SetString(PyExc_RuntimeError, "could not initialise gnomevfs");
 	return;
@@ -2509,7 +2445,7 @@ initgnomevfs(void)
     if (PyType_Ready(&PyGnomeVFSXferProgressInfo_Type) < 0)
 	return;
 
-    m = Py_InitModule("gnomevfs", pygnomevfs_functions);
+    m = Py_InitModule("gnomevfs._gnomevfs", pygnomevfs_functions);
     d = PyModule_GetDict(m);
 
     register_constants(m);
